%Normalized Logarithmic Binning Estimator

%Estimates the exponent of power-laws using normalized logarithmic binning.
%This function allows for a variety of modifications including the presence
%of a known minimum value of x (minval_present = 1), the specification of
%the minimum number of points required to include a bin in the
%regression(minpts), and the ability to drop the last bin from the analysis
%(droplastbin=1). The minimum value of x is accomodated by starting the
%edge of the first bin at that minimum value, thus avoiding underestimating
%the frequency in the first bin. Dropping the last bin can avoid the
%related problem of underestimating the frequency in the largest bin if
%there is a meaningful maximum value of x. In most analyses minpts is set
%to 1, but allowing the specification of this value allows the dropping of
%singleton bins as recommended by Pickering et al. 1995 (since bins with
%fractional densities will most commonly be realized as zeros or ones and
%dropping only the zeros can lead to biased estimates) and also allows the
%replication of analyses where greater values of minpts have been used
%(e.g., Enquist and Niklas 2001). See the text of the paper for citation
%details. 'data' is a vector of x values. This function uses the natural
%logarithmic.

function [normexponent] = logbin_estimator(data,binwidth,minval_present,minval,droplastbin,minpts)

%Bin the data using the provided bin width and starting the first bin edge
%either at the default position for a histogram or at the provided minimum
%value, and determine the centers of the bins.
if minval_present==1
    %Determine the upper edge of the last bin
    upperedge=log(minval)+binwidth*ceil((max(log(data))-log(minval))/binwidth);
    %Bin the data using the 'histc' function
    n=histc(log(data),log(minval):binwidth:upperedge);
    %Include any data point equal to the upper edge in the last bin and
    %remove the bin that original caught these data.
    n(length(n)-1)=n(length(n)-1)+n(length(n));n(length(n))=[];
    %Determine the bin centers
    centers=[(log(minval)+.5*binwidth):binwidth:(upperedge-.5*binwidth)]';
else
    %Or simple retrieve n and centers using 'hist' if no minimum is present
    [n,centers]=hist(log(data));
end

%Determine the linear width of the bins
linearwidthofbins=exp(centers+.5*binwidth)-exp(centers-.5*binwidth);

%Drop the last bin if instructed
if droplastbin==1
    centers(length(centers))=[];
    linearwidthofbins(length(linearwidthofbins))=[];
    n(length(n))=[];
end

%Drop bins with less thean the minimum number of points for analysis
centers=centers(n>=minpts);
linearwidthofbins=linearwidthofbins(n>=minpts);
n=n(n>=minpts);

%Normalize the frequency data
n=n./linearwidthofbins;

%Fit an OLS regression to the log transformed data. Note that since the
%centers are already on a logarithmic scale they are not transformed again
normparas=polyfit(centers,log(n),1);

%Return the slope of the above regression as an estimate of the exponent
normexponent=normparas(:,1);